//
// Copyright 2006 Mentor Graphics Corporation
//
// All Rights Reserved.
//
// THIS WORK CONTAINS TRADE SECRET AND PROPRIETARY INFORMATION WHICH IS THE PROPERTY
// OF MENTOR GRAPHICS CORPORATION OR ITS LICENSORS AND IS SUBJECT TO LICENSE TERMS.
//

`timescale 1 ns / 1 ns

module proc(clk, addr, data, rw, strb, rdy);
    input  clk, rdy;
    output addr, rw, strb;
    inout  data;

    `define addr_size 8
    `define word_size 16

    logic [`addr_size-1:0] addr_r;
    logic [`word_size-1:0] data_r;
    logic                  rw_r, strb_r;

    wire [`addr_size-1:0] #(5) addr = addr_r;
    wire [`word_size-1:0] #(5) data = data_r;
    wire                  #(5) rw = rw_r, strb = strb_r;

    enum {IDLE, READ, WRITE} state;

    task read;
        input  [`addr_size-1:0] a;
        logic [`word_size-1:0] d;
        begin
            addr_r = a;
            rw_r = 1;
            strb_r = 0;
            @(posedge clk) strb_r = 1;
            while (rdy != 0) @(posedge clk) ;
            d = data;
            if (top.verbose)
                $display("%t: Reading data=%d from addr=%d", $time, d, a);
        end
    endtask
    
    task write;
        input  [`addr_size-1:0] a;
        input  [`word_size-1:0] d;
        begin
            if (top.verbose)
                $display("%t: Writing data=%d to addr=%d", $time, d, a);
            addr_r = a;
            rw_r = 0;
            strb_r = 0;
            data_r = d;
            @(posedge clk) strb_r = 1;
            while (rdy != 0) @(posedge clk) ;
            data_r = 'bz;
            @(posedge clk);
        end
    endtask

    /**************** _` START ****************/

    class rand_sig;
        rand bit [7:0] a;
        rand bit [15:0] d;
        constraint d_range {
            solve a before d;
            (a < 128) -> d inside {[0:9999]};
            (a >= 128) -> d inside {[10000:19999]};}
    endclass

    rand_sig rand_i = new;

    integer rep;

    /**************** _ϐ END ****************/

    initial begin
        // Set initial state of outputs..
        addr_r = 0;
        data_r = 'bz;
        rw_r = 0;
        strb_r = 1;
        
        // Wait for first clock, then perform read/write test
        @(posedge clk)
        if (top.verbose)
            $display("%t: Starting Read/Write test", $time);

    /**************** _p^[ START ****************/

//    rand_i.srandom(100);

    repeat (100) begin

        randsequence (main)
        main: IDLE RDWR;
        RDWR: READ | WRITE := 0;

        READ: {
            randcase
                1: rep = 3; 2: rep = 2; 3: rep = 1;
            endcase
           if (top.verbose)
               $display ("-- READ  x %3d times", rep);
            state = READ;
            repeat (rep) begin
                if (rand_i.randomize() == 1)
                    read(rand_i.a);
                else
                    $display ("Random Failed");
            end
            };

        WRITE: {
            randcase
                1: rep = 3; 2: rep = 2; 3: rep = 1;
            endcase
           if (top.verbose)
                $display ("-- WRITE  x %3d times", rep);
            state = WRITE;
            repeat (rep) begin
                if (rand_i.randomize() == 1)
                    write(rand_i.a, rand_i.d);
                else
                    $display ("Random Failed");
            end
            };

        IDLE: {
            randcase
                1: rep = 10; 2: rep = 5; 3: rep = 3;
            endcase
           if (top.verbose)
                $display ("-- IDLE  x %3d cycle", rep);
            state = IDLE;
            repeat (rep) @ (posedge clk);
            };

        endsequence
    end


    /**************** _p^[ END ****************/

        if (top.verbose)
            $display("Randome Read/Write test done");
        $stop(1);

    end // initial

endmodule
