//
// Copyright 2006 Mentor Graphics Corporation
//
// All Rights Reserved.
//
// THIS WORK CONTAINS TRADE SECRET AND PROPRIETARY INFORMATION WHICH IS THE PROPERTY
// OF MENTOR GRAPHICS CORPORATION OR ITS LICENSORS AND IS SUBJECT TO LICENSE TERMS.
//

`timescale 1 ns / 1 ns

interface pmbus (input bit CLK);
    logic [7:0] ADDR;
    wire [15:0] DATA;
    wire RW;
    wire STRB;
    wire RDY;

    modport bus_p (
        input CLK,
        inout DATA,
        output ADDR, RW, STRB,
        input RDY);

    modport bus_m (
        input CLK,
        inout DATA,
        input ADDR, RW, STRB,
        output RDY);

    /*********** 3̓AT[V **********/

    sequence RdStrb;
        !STRB ##1 STRB && RW;
    endsequence

    property RdRdy;
        @(posedge CLK) RdStrb |=> STRB[*0:8] ##0 !RDY;
    endproperty

    assert property (RdRdy);

    a_01: assert property (RdRdy);

    assert property (@(posedge CLK) RdStrb |=> STRB[*0:8] ##0 !RDY);

    /*********** ̑̃AT[V **********/

    property StrbOnce;
        @ (posedge CLK) !STRB |=> STRB;
    endproperty

    assert property (StrbOnce);

    property DataValid;
        @ (posedge CLK) (!STRB && !RW) |=> !$isunknown(DATA);
    endproperty

    assert property (DataValid);

    property BusIdle;
        @ (posedge CLK) !RDY |=> RDY && (DATA === 16'hzzzz);
    endproperty

    assert property (BusIdle);

    /*********** cover fBNeBu **********/

    sequence WrStart;
        !STRB && !RW ##1 STRB;
    endsequence

    sequence WrData;
        (!$isunknown(DATA) && STRB)[*0:8];
    endsequence

    sequence WrDone;
        !RDY ##1 RDY && (DATA === 16'hzzzz);
    endsequence

    sequence WriteCycle;
        @ (posedge CLK) WrStart ##1 WrData ##0 WrDone;
    endsequence

    cover property (WriteCycle);

endinterface

module top;

  logic clk;

  logic verbose = 1;

  // Processor side signals
  logic prw, pstrb, prdy;
  logic [7:0] paddr;
  wire [15:0] pdata;

  // Memory side interface
  pmbus pm0 (.CLK(clk));

  initial clk = 1'b0;
  always #20 clk = ~clk;

  proc p (.clk(clk), .addr(paddr), .data(pdata), 
          .rw(prw), .strb(pstrb), .rdy(prdy));

//  cache_wb c (.paddr(paddr), .pdata(pdata),
  cache c (.paddr(paddr), .pdata(pdata),
           .prw(prw), .pstrb(pstrb), .prdy(prdy),
           .p0(pm0));

  memory m (.m0(pm0));
endmodule